# frozen_string_literal: true

require 'json'

module Docker2plantuml
  class Command
    def self.run(argv)
      new(argv).run
    end

    def initialize(argv)
      @argv = argv
    end

    def run
      json_data = if stdin?
                    JSON.parse(STDIN.read)
                  else
                    JSON.parse(File.read(@argv[0]))
                  end
      containers = Docker2plantuml::DockerJsonParser.new(json_data).parse
      w = Docker2plantuml::PlantUMLWriter.new(containers)
      puts Docker2plantuml::HEADER
      puts w.to_plant_uml
    end

    def stdin?
      File.pipe?(STDIN) || !File.select([STDIN], [], [], 0).nil?
    end
  end
end
