# frozen_string_literal: true

module Docker2plantuml
  class DockerJsonParser
    def initialize(json)
      @json_data = json
    end

    def parse
      @json_data.map do |json|
        container = parse_container(json)
        parse_network_and_port(json, container)
        container
      end
    end

    def parse_container(json)
      Docker2plantuml::DockerContainer.new(
        parse_container_name(json),
        parse_container_status(json)
      )
    end

    def parse_container_name(json)
      name = json['Name']
      name.slice(1..-1) if name.start_with?('/')
    end

    def parse_container_status(json)
      json['State']['Status']
    end

    def parse_network_and_port(json, container)
      networks = parse_networks(json)
      return unless networks.is_a? String

      container.network_setting = Docker2plantuml::NetworkSetting.new(networks)
      container.port_info_list.concat(parse_ports(json))
    end

    def parse_networks(json)
      network_settings = parse_network_settings(json)
      network_settings['Networks'].map { |k, _v| k }.first
    end

    def parse_network_settings(json)
      json['NetworkSettings']
    end

    def parse_ports(json)
      network_settings = parse_network_settings(json)
      create_port_info(network_settings['Ports'])
    end

    def create_port_info(ports)
      ret = ports
            .reject { |_container_port, host_port| host_port.nil? }
            .map do |c_port, h_port|
              h_port.map { |val| Docker2plantuml::PortInfo.new(c_port, val['HostPort']) }
            end
      ret = ret.inject([]) { |list, p| list + p }
      ret.concat(
        ports.select { |_container_port, host_port| host_port.nil? }
             .map { |container_port, _host_port| Docker2plantuml::PortInfo.new(container_port) }
      )
    end
  end
end
