# frozen_string_literal: true

module Docker2plantuml
  class PlantUMLWriter
    I = '  '
    II = '    '
    III = '      '

    def initialize(container_list)
      @container_list = container_list
    end

    def to_plant_uml
      [
        '@startuml',
        'folder DockerHost {',
        host_ports_to_plant_uml,
        containers_to_plant_uml,
        port_mappings_to_plant_uml,
        '}',
        '@enduml'
      ].join("\n")
    end

    def host_ports_to_plant_uml
      @container_list
        .map(&:server_port_list)
        .reject(&:empty?)
        .inject([]) { |list, item| list + item }
        .map { |p| I + "[#{p}]" }
        .join("\n")
    end

    def containers_to_plant_uml
      @container_list
        .map { |c| container_to_plant_uml(c) }
        .join("\n")
    end

    def container_to_plant_uml(container)
      container_name = container.container_name
      ports = ports_to_plant_uml(container)
      [
        I + "cloud [#{container.network_setting.network_name}\\n\\n]{",
        II + "node [#{container_name}\\n]{",
        ports.empty? ? III + '[ ]' : ports,
        II + '}',
        I + '}'
      ].join("\n")
    end

    def ports_to_plant_uml(container)
      container_name = container.container_name
      container
        .port_info_list
        .map { |p| III + "[#{container_name}:#{p.container_port}]" }
        .join("\n")
    end

    def port_mappings_to_plant_uml
      @container_list
        .map { |c| port_mapping_to_plant_uml(c) }
        .reject(&:empty?)
        .join("\n")
    end

    def port_mapping_to_plant_uml(container)
      container_name = container.container_name
      return '' if container.port_info_list.empty?

      container
        .port_info_list
        .select(&:host_port)
        .map { |p| I + port_mapping(container_name, p) }
        .join("\n")
    end

    def port_mapping(c_name, p_info)
      "[#{c_name}:#{p_info.container_port}] -- [#{p_info.host_port}]"
    end
  end
end
